// -
//   ========================LICENSE_START=================================
//   O-RAN-SC
//   %%
//   Copyright (C) 2022: Nordix Foundation
//   %%
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//        http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//   ========================LICENSE_END===================================
//

package org.oransc.nonrtric.plt.capifcore.controller;

import org.oransc.capif.invokermanagement.server.api.OnboardedInvokersApi;
import org.oransc.capif.invokermanagement.server.api.OnboardedInvokersApi;
import org.oransc.capif.model.APIInvokerEnrolmentDetails;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import java.net.URI;

@RestController
@RequestMapping("api-invoker-management/v1")
public class InvokerManager implements OnboardedInvokersApi {
    private final ApiRegister apiRegister;
    private int nextId;

    public InvokerManager(ApiRegister apiRegister) {
        this.apiRegister = apiRegister;
    }

    @Override
    public ResponseEntity<APIInvokerEnrolmentDetails> onboardedInvokersPost(APIInvokerEnrolmentDetails apIInvokerEnrolmentDetails) {
        apIInvokerEnrolmentDetails.apiInvokerId(getApiId(apIInvokerEnrolmentDetails.getApiInvokerInformation()));

        URI uri = ServletUriComponentsBuilder
                .fromCurrentRequest()
                .path("/{id}")
                .buildAndExpand(apIInvokerEnrolmentDetails.getApiInvokerId())
                .toUri();
        return ResponseEntity.created(uri).body(apIInvokerEnrolmentDetails);
    }

    private String getApiId(String invokerInfo) {
        String idAsString = "api_invoker_id_";

        if (invokerInfo != null && !invokerInfo.isEmpty()) {
            idAsString = idAsString + invokerInfo.replaceAll("\\s+", "_");
        } else {
            idAsString = idAsString + nextId++;
        }
        return idAsString;
    }
}
